@@ -1,171 +1,13 @@
 from __future__ import generator_stop
 
 from operator import itemgetter
-from typing import TYPE_CHECKING, Any, Callable, Dict, Iterable, Iterator, List, Optional, Set, Tuple, TypeVar, Union
+from typing import Callable, Iterator, Optional, Set, Tuple, TypeVar, Union
 
-if TYPE_CHECKING:
-    T = TypeVar("T")
-
-    from graphviz import Digraph
-
-
-class BKNode:
-
-    __slots__ = ["value", "leaves"]
-
-    def __init__(self, value, leaves):
-        # type: (Any, Dict[Any, BKNode]) -> None
-
-        self.value = value
-        self.leaves = leaves
-
-    def __str__(self):
-        return str(self.value) + ": " + str(self.leaves)
-
-    def __repr__(self):
-        return repr(self.value) + ": " + repr(self.leaves)
-
-
-class BKTree:
-
-    """BK-tree. Used for nearest neighbor queries according to a discrete metric.
-    Examples for the metric are the Manhattan distance or the Levenshtein distance.
-
-    from Levenshtein import distance # pip install python-Levenshtein
-
-    words = ["laptop", "security", "microsoft", "computer", "software", "tree", "algorithm", "desktop"]
-
-    tree = BKTree(distance)
-    tree.update(words)
-    tree.saveimage("bk-tree.gv")
-    """
-
-    def __init__(self, distance_func):
-        # type: (Callable[[Any, Any], int], ) -> None
-
-        self.distance_func = distance_func
-        self.root = None  # type: Optional[BKNode]
-
-    def add(self, value):
-        # type: (Any, ) -> None
-
-        if self.root is None:
-            self.root = BKNode(value, {})
-            return
-
-        node = self.root
-        while True:
-            distance = self.distance_func(value, node.value)
-            try:
-                node = node.leaves[distance]
-            except KeyError:
-                node.leaves[distance] = BKNode(value, {})
-                break
-
-    def update(self, values):
-        # type: (Iterable[Any], ) -> None
-
-        for value in values:
-            self.add(value)
-
-    def find(self, value, max_distance):
-        # type: (Any, int) -> List[Tuple[int, Any]]
-
-        """Returns all values from tree where the metric distance
-        is less or equal to `max_distance`.
-        """
-
-        node = self.root
-        ret = []  # type: List[Tuple[int, Any]]
-
-        if node is None:
-            return ret
-
-        candidates = [node]  # is a deque better here?
-
-        while candidates:
-            candidate = candidates.pop()
-            distance = self.distance_func(value, candidate.value)
-
-            if distance <= max_distance:
-                ret.append((distance, candidate.value))
-
-            # instead of looking for candidates by searching,
-            # one could also directly access the necessary keys in the dict
-            for d, bknode in candidate.leaves.items():
-                lower = distance - max_distance
-                upper = distance + max_distance
-                if lower <= d <= upper:
-                    candidates.append(bknode)
-
-        return ret
-
-    @staticmethod
-    def _find_by_distance(node, distance):
-        # type: (BKNode, int) -> Iterator[Set[Any]]
-
-        for d, bknode in node.leaves.items():
-            if d == distance:
-                nodeset = set(BKTree._values(bknode))
-                nodeset.add(node.value)
-                yield nodeset
-            yield from BKTree._find_by_distance(bknode, distance)
-
-    def find_by_distance(self, distance):
-        # type: (int, ) -> Iterator[Set[Any]]
-
-        """Find all sets of values base on `distance` between each other."""
-
-        if self.root is None:
-            return iter([])
-        else:
-            return BKTree._find_by_distance(self.root, distance)
-
-    @staticmethod
-    def _dot(dot, node):
-        # type: (Digraph, BKNode) -> None
-
-        for distance, childnode in node.leaves.items():
-            dot.node(str(childnode.value))
-            dot.edge(str(node.value), str(childnode.value), label=str(distance))
-            BKTree._dot(dot, childnode)
-
-    def saveimage(self, filename, format="png"):
-        from graphviz import Digraph
-
-        if self.root is None:
-            raise ValueError("Tree is empty")
-
-        dot = Digraph(format=format)
-        dot.node(str(self.root.value))
-        self._dot(dot, self.root)
-        dot.render(filename)
-
-    @staticmethod
-    def _values(node):
-        # type: (BKNode, ) -> Iterator[Any]
-
-        yield node.value
-        for leaf in node.leaves.values():
-            yield from BKTree._values(leaf)
-
-    def values(self):
-        # type: () -> Iterator[Any]
-
-        if self.root is None:
-            return iter([])
-        else:
-            return self._values(self.root)
-
-    def __iter__(self):
-        # type: () -> Iterator[Any]
-
-        return self.values()
+T = TypeVar("T")
 
 
 class RoutingObject:
-    def __init__(self, value):
-        # type: (T, ) -> None
+    def __init__(self, value: T) -> None:
 
         self.value = value
 
@@ -173,21 +15,20 @@ class RoutingObject:
         self.distance_to_parent = None  # type: Optional[float]
         self.covering_radius = None  # type: Optional[float]
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         return f"RoutingObject({self.value}, radius={self.covering_radius}, tree={self.subtree})"
 
-    def set_subtree(self, node):
+    def set_subtree(self, node) -> None:
         self.subtree = node
         node.parent_object = self
 
 
 class LeafObject:
-    def __init__(self, value):
-        # type: (T, ) -> None
+    def __init__(self, value: T) -> None:
 
         self.value = value
 
-        self.distance_to_parent = None  # type: Optional[float]
+        self.distance_to_parent: Optional[float] = None
 
     def __repr__(self):
         return f"LeafObject({self.value})"
