@@ -1,7 +1,8 @@
 from __future__ import generator_stop
 
 from hashlib import sha1
-from typing import TYPE_CHECKING, Any, Dict, Iterator, Optional, Union
+from pathlib import Path
+from typing import Any, Dict, Iterable, Iterator, Optional
 
 import bencode
 
@@ -9,32 +10,23 @@ from .exceptions import assert_choice
 from .file import blockfileiter, blockfilesiter
 from .filesystem import FileProperties
 
-if TYPE_CHECKING:
-    from pathlib import Path
 
-    PathType = Union[str, Path]
-
-
-def read_torrent(path):
-    # type: (str, ) -> dict
+def read_torrent(path: str) -> dict:
 
     return bencode.bread(path)
 
 
-def write_torrent(data, path):
-    # type: (dict, PathType) -> None
+def write_torrent(data: dict, path: str) -> None:
 
     bencode.bwrite(data, path)
 
 
-def read_torrent_info_dict(path):
-    # type: (PathType, ) -> dict
+def read_torrent_info_dict(path: str) -> dict:
 
     return bencode.bread(path)["info"]
 
 
-def iter_torrent(path):
-    # type: (PathType, ) -> Iterator[FileProperties]
+def iter_torrent(path: str) -> Iterator[FileProperties]:
 
     """Returns file informations from a torrent file.
     Hash (SHA-1) is obtained according to BEP0047 (if available).
@@ -52,12 +44,11 @@ def iter_torrent(path):
             yield FileProperties(info["name"] + "/" + path, fd["length"], False, hash=fd.get("sha1"))
 
 
-def pieces_field(pieces):
+def pieces_field(pieces: Iterable[bytes]) -> bytes:
     return b"".join(sha1(piece).digest() for piece in pieces)  # nosec
 
 
-def create_torrent_info_dict(path, piece_size, private=None):
-    # type: (Path, int, Optional[int]) -> Dict[str, Any]
+def create_torrent_info_dict(path: Path, piece_size: int, private: Optional[int] = None) -> Dict[str, Any]:
 
     if private is not None:
         assert_choice("private", private, {0, 1})
@@ -97,16 +88,15 @@ def create_torrent_info_dict(path, piece_size, private=None):
     return ret
 
 
-def torrent_info_hash(d):
+def torrent_info_hash(d: dict) -> str:
     return sha1(bencode.bencode(d)).hexdigest()  # nosec
 
 
-def get_torrent_hash(path):
+def get_torrent_hash(path: str) -> str:
     return torrent_info_hash(read_torrent_info_dict(path))
 
 
-def create_torrent(path, piece_size, announce=""):
-    # type: (Path, int, str) -> bytes
+def create_torrent(path: Path, piece_size: int, announce: str = "") -> bytes:
 
     info = create_torrent_info_dict(path, piece_size)
     torrent = {"info": info, "announce": announce}
